"""

"""
import numpy as np
import numpy.linalg as la

def eigenvector_derivatives(H0, S0, dHdp0, dSdp0):
    # compute eigenvectors and eigenvalues
    E,X = la.eigh(H0)
    print "E = "
    print E
    Xt = X.conjugate().transpose()
    ndim = len(H0)
    # gradient of eigenvalues
    dEdp = np.zeros(ndim)
    # gradient of eigenvectors
    C = np.zeros((ndim,ndim))
    for i in range(0, ndim):
        dEdp[i] = np.dot(Xt[i,:], np.dot(dHdp0, X[:,i]))
        for j in range(0, ndim):
            if i != j:
                C[i,j] = np.dot(Xt[i,:], np.dot(dHdp0, X[:,j]))/(E[j]-E[i])
            else:
                C[i,i] = 0.0
    dXdp = np.dot(X, C)
    return dEdp, dXdp

def H(p):
    a11 = 2*np.cos(p)*(-2+2*p**2+(1+p**2)*np.cos(2*p)-4*p**2*np.sin(p))
    a12 = 8*p*(-2*np.cos(p)+np.sin(2*p))
    a13 = (-3+5*p**2)*np.cos(p)-(1+p**2)*np.cos(3*p)-4*p**2*np.sin(2*p)
    a21 = 4*p*(-2*np.cos(p)+np.sin(2*p))
    a22 = -2*np.cos(p)*(-5+3*p**2+8*np.sin(p))
    a23 = 4*p*(-2*np.cos(p)+np.sin(2*p))
    a31 = (-3+5*p**2)*np.cos(p)-(1+p**2)*np.cos(3*p)-4*p**2*np.sin(2*p)
    a32 = 8*p*(-2*np.cos(p)+np.sin(2*p))
    a33 = 2*np.cos(p)*(-2+2*p**2+(1+p**2)*np.cos(2*p)-4*p**2*np.sin(p))
    h = np.array(\
        [[a11,a12,a13],
         [a12,a22,a23],
         [a13,a23,a33]])
    h /= (2*(1+p**2))
    return h

def S(p):
    s = np.array(\
        [[1,0,0],
         [0,1,0],
         [0,0,1]])
    return s

dp = 0.0001                        

def dHdp(p):
    h0 = H(p)
    h1 = H(p+dp)
    dh = (h1-h0)/dp
    return dh

def dSdp(p):
    s0 = S(p)
    s1 = S(p+dp)
    ds = (s1-s0)/dp
    return ds

def dXdp(p):
    E0,X0 = la.eigh(H(p))
    E1,X1 = la.eigh(H(p+dp))
    dE = (E1-E0)/dp
    dX = (X1-X0)/dp
    return dE, dX

def plot_eigenvalues():
    from matplotlib import pyplot as plt
    ps = np.linspace(0.0, 2.0*np.pi, 2000)
    Es = []
    dEs = []
    dEs_an = []
    for i,p in enumerate(ps):
        Hp = H(p)
        e,v = la.eigh(Hp)
        Es.append(e)
        # numeric
        de,dX = dXdp(p)
        dEs.append(de)
        # analytic
        de,dX = eigenvector_derivatives(Hp, S(p), dHdp(p), dSdp(p))
        dEs_an.append(de)
    Es = np.array(Es)
    dEs = np.array(dEs)
    dEs_an = np.array(dEs_an)
    ndim = len(Hp)
    for n in range(0, ndim):
        plt.plot(ps, Es[:,n], label="%d" % n)
        plt.plot(ps, dEs[:,n], ls="-.", label="%d" % n)
        plt.plot(ps, dEs_an[:,n], "x", label="%d" % n)
    plt.legend()
    plt.show()


if __name__ == "__main__":
    plot_eigenvalues()

    p0 = 4.71 #1.57 #4.71
    H0, S0 = H(p0), S(p0)
    print H0
    dHdp0, dSdp0 = dHdp(p0), dSdp(p0)
    dE,dX = dXdp(p0)
    print "dE (numeric)"
    print dE
    print "dX (numeric)"
    print dX
    dE_analytic, dX_analytic = eigenvector_derivatives(H0, S0, dHdp0, dSdp0)
    print "dE (analytic)"
    print dE_analytic
    print "dX (analytic)"
    print dX_analytic

