! This file has been generated automatically


! transformation rules for matrix elements of dipole operator
DOUBLE PRECISION FUNCTION slako_transformation_dipole(r,x,y,z, Dip, N, l1,m1, mr, l2,m2)
   IMPLICIT NONE
   ! x,y,z are directional cosines, r is the distance between the two centers
   DOUBLE PRECISION, INTENT(IN) :: r,x,y,z
   INTEGER, INTENT(IN) :: N  ! length of array Dip
   ! values of the N Slater-Koster tables for the dipole evaluated at distance r
   DOUBLE PRECISION, INTENT(IN) :: Dip(N)
   ! orbital qm numbers for center 1 and center 2
   INTEGER, INTENT(IN) :: l1,m1,l2,m2
   ! qm number for orientation of r-vector, -1: y, 0: z, 1: x
   INTEGER, INTENT(IN) :: mr
   ! Local Variables
   INTEGER, PARAMETER :: lmax=1
   ! Result D(x,y,z)=<l1,m1|r_(lr,mr)|l2,m2> after applying SK rules
   DOUBLE PRECISION :: res
   INTEGER :: i  ! index that encodes the tuple (l1,m1,mr,l2,m2)

   ! First we need to transform the tuple (l1,m1,mr,l2,m2) into a unique integer
   ! so that the compiler can build a branching table for each case.
   ! Valid ranges for qm numbers: 0 <= l1,l2 <= lmax, -lmax <= m1,m2 <= lmax and -1 <= mr <= 1
   i = (mr+1)*10000+l1*1000+(lmax+m1)*100+l2*10+(lmax+m2)
   SELECT CASE(i)
	CASE (101)
	! (0,0,-1,0,0)
	 res = y*Dip(3)
	CASE (110)
	! (0,0,-1,1,-1)
	 res = (x**2 + z**2)*Dip(2) + y**2*Dip(4)
	CASE (111)
	! (0,0,-1,1,0)
	 res = y*z*(-Dip(2) + Dip(4))
	CASE (112)
	! (0,0,-1,1,1)
	 res = x*y*(-Dip(2) + Dip(4))
	CASE (10101)
	! (0,0,0,0,0)
	 res = z*Dip(3)
	CASE (10110)
	! (0,0,0,1,-1)
	 res = y*z*(-Dip(2) + Dip(4))
	CASE (10111)
	! (0,0,0,1,0)
	 res = (x**2 + y**2)*Dip(2) + z**2*Dip(4)
	CASE (10112)
	! (0,0,0,1,1)
	 res = x*z*(-Dip(2) + Dip(4))
	CASE (20101)
	! (0,0,1,0,0)
	 res = x*Dip(3)
	CASE (20110)
	! (0,0,1,1,-1)
	 res = x*y*(-Dip(2) + Dip(4))
	CASE (20111)
	! (0,0,1,1,0)
	 res = x*z*(-Dip(2) + Dip(4))
	CASE (20112)
	! (0,0,1,1,1)
	 res = (y**2 + z**2)*Dip(2) + x**2*Dip(4)
	CASE (1001)
	! (1,-1,-1,0,0)
	 res = (x**2 + z**2)*Dip(5) + y**2*Dip(8)
	CASE (1010)
	! (1,-1,-1,1,-1)
	 res = y*(x**2 + z**2)*(Dip(6) + 2*Dip(7)) + y**3*Dip(9)
	CASE (1011)
	! (1,-1,-1,1,0)
	 res = z*((x**2 + z**2)*Dip(6) + y**2*(-2*Dip(7) + Dip(9)))
	CASE (1012)
	! (1,-1,-1,1,1)
	 res = x*((x**2 + z**2)*Dip(6) + y**2*(-2*Dip(7) + Dip(9)))
	CASE (11001)
	! (1,-1,0,0,0)
	 res = y*z*(-Dip(5) + Dip(8))
	CASE (11010)
	! (1,-1,0,1,-1)
	 res = z*(-(y**2*Dip(6)) + (x**2 - y**2 + z**2)*Dip(7) + y**2*Dip(9))
	CASE (11011)
	! (1,-1,0,1,0)
	 res = y*((x**2 + y**2)*Dip(7) - z**2*(Dip(6) + Dip(7) - Dip(9)))
	CASE (11012)
	! (1,-1,0,1,1)
	 res = x*y*z*(-Dip(6) - 2*Dip(7) + Dip(9))
	CASE (21001)
	! (1,-1,1,0,0)
	 res = x*y*(-Dip(5) + Dip(8))
	CASE (21010)
	! (1,-1,1,1,-1)
	 res = x*(-(y**2*Dip(6)) + (x**2 - y**2 + z**2)*Dip(7) + y**2*Dip(9))
	CASE (21011)
	! (1,-1,1,1,0)
	 res = x*y*z*(-Dip(6) - 2*Dip(7) + Dip(9))
	CASE (21012)
	! (1,-1,1,1,1)
	 res = y*(-(x**2*Dip(6)) + (-x**2 + y**2 + z**2)*Dip(7) + x**2*Dip(9))
	CASE (1101)
	! (1,0,-1,0,0)
	 res = y*z*(-Dip(5) + Dip(8))
	CASE (1110)
	! (1,0,-1,1,-1)
	 res = z*(-(y**2*Dip(6)) + (x**2 - y**2 + z**2)*Dip(7) + y**2*Dip(9))
	CASE (1111)
	! (1,0,-1,1,0)
	 res = y*((x**2 + y**2)*Dip(7) - z**2*(Dip(6) + Dip(7) - Dip(9)))
	CASE (1112)
	! (1,0,-1,1,1)
	 res = x*y*z*(-Dip(6) - 2*Dip(7) + Dip(9))
	CASE (11101)
	! (1,0,0,0,0)
	 res = (x**2 + y**2)*Dip(5) + z**2*Dip(8)
	CASE (11110)
	! (1,0,0,1,-1)
	 res = y*((x**2 + y**2)*Dip(6) + z**2*(-2*Dip(7) + Dip(9)))
	CASE (11111)
	! (1,0,0,1,0)
	 res = (x**2 + y**2)*z*(Dip(6) + 2*Dip(7)) + z**3*Dip(9)
	CASE (11112)
	! (1,0,0,1,1)
	 res = x*((x**2 + y**2)*Dip(6) + z**2*(-2*Dip(7) + Dip(9)))
	CASE (21101)
	! (1,0,1,0,0)
	 res = x*z*(-Dip(5) + Dip(8))
	CASE (21110)
	! (1,0,1,1,-1)
	 res = x*y*z*(-Dip(6) - 2*Dip(7) + Dip(9))
	CASE (21111)
	! (1,0,1,1,0)
	 res = x*((x**2 + y**2)*Dip(7) - z**2*(Dip(6) + Dip(7) - Dip(9)))
	CASE (21112)
	! (1,0,1,1,1)
	 res = z*(-(x**2*Dip(6)) + (-x**2 + y**2 + z**2)*Dip(7) + x**2*Dip(9))
	CASE (1201)
	! (1,1,-1,0,0)
	 res = x*y*(-Dip(5) + Dip(8))
	CASE (1210)
	! (1,1,-1,1,-1)
	 res = x*(-(y**2*Dip(6)) + (x**2 - y**2 + z**2)*Dip(7) + y**2*Dip(9))
	CASE (1211)
	! (1,1,-1,1,0)
	 res = x*y*z*(-Dip(6) - 2*Dip(7) + Dip(9))
	CASE (1212)
	! (1,1,-1,1,1)
	 res = y*(-(x**2*Dip(6)) + (-x**2 + y**2 + z**2)*Dip(7) + x**2*Dip(9))
	CASE (11201)
	! (1,1,0,0,0)
	 res = x*z*(-Dip(5) + Dip(8))
	CASE (11210)
	! (1,1,0,1,-1)
	 res = x*y*z*(-Dip(6) - 2*Dip(7) + Dip(9))
	CASE (11211)
	! (1,1,0,1,0)
	 res = x*((x**2 + y**2)*Dip(7) - z**2*(Dip(6) + Dip(7) - Dip(9)))
	CASE (11212)
	! (1,1,0,1,1)
	 res = z*(-(x**2*Dip(6)) + (-x**2 + y**2 + z**2)*Dip(7) + x**2*Dip(9))
	CASE (21201)
	! (1,1,1,0,0)
	 res = (y**2 + z**2)*Dip(5) + x**2*Dip(8)
	CASE (21210)
	! (1,1,1,1,-1)
	 res = y*((y**2 + z**2)*Dip(6) + x**2*(-2*Dip(7) + Dip(9)))
	CASE (21211)
	! (1,1,1,1,0)
	 res = z*((y**2 + z**2)*Dip(6) + x**2*(-2*Dip(7) + Dip(9)))
	CASE (21212)
	! (1,1,1,1,1)
	 res = x*(y**2 + z**2)*(Dip(6) + 2*Dip(7)) + x**3*Dip(9)
  CASE DEFAULT
	 WRITE(*,*) "BUG: No case for i=",i," which encodes (l1,m1,mr,l2,m2)=(",l1,",",m1,",",mr,",",l2,",",m2,") !"
	 STOP
   END SELECT
   slako_transformation_dipole = res
END FUNCTION
